@@ -5,14 +5,14 @@ import nox
 nox.options.reuse_existing_virtualenvs = True
 
 
-@nox.session
+@nox.session(python=["3.7", "3.8", "3.9", "3.10", "3.11"])
 def lint(session: nox.Session) -> None:
     session.install("pre-commit")
     session.run("pre-commit", "install")
     session.run("pre-commit", "run", "--all-files")
 
 
-@nox.session(python=["3.9"])
+@nox.session(python=["3.7", "3.8", "3.9", "3.10", "3.11"])
 def build(session):
     session.install(".[dev,test]")
     session.run(
