@@ -11,10 +11,11 @@ This is the complete API reference:
 
    read
    ReadFCS
+   view
    datasets
 """
 
-__version__ = "1.0.4"
+__version__ = "1.1.0"
 
 # prints warning of python versions
 from lamin_logger import py_version_warning
@@ -22,4 +23,4 @@ from lamin_logger import py_version_warning
 py_version_warning("3.7", "3.10")
 
 from . import datasets
-from ._core import ReadFCS, read
+from ._core import ReadFCS, read, view
